@@ -1,7 +1,7 @@
 use crate::prelude::*;
 use gdnative::prelude::*;
-use hashbrown::HashMap;
 
+/// The primary Godot interface to the database.
 #[derive(NativeClass)]
 #[inherit(Node)]
 pub struct GDDB {
@@ -15,6 +15,7 @@ impl GDDB {
         Self { storage: db }
     }
 
+    // Creates a database record
     #[export]
     pub fn create(&mut self, _owner: &Node, model: String, attributes: Dictionary) -> String {
         let mut record = Record::new(model);
@@ -26,18 +27,32 @@ impl GDDB {
         uuid
     }
 
+    // Finds a database record given a uuid
     #[export]
-    pub fn find(&mut self, _owner: &Node, uuid: String) -> Dictionary<Unique> {
+    pub fn find(&mut self, _owner: &Node, uuid: String) -> GodotString {
         let record = self.storage.find(|f| &f.uuid, uuid).unwrap();
-        let json: HashMap<String, String> =
-            serde_json::from_str(&record.attributes.clone()).unwrap();
 
         let data = Dictionary::new();
 
-        for (key, value) in json {
-            data.insert(key, value);
-        }
+        data.insert("uuid", record.uuid.clone());
+        data.insert("model", record.model.clone());
+        data.insert("attributes", record.attributes.clone());
 
-        data
+        data.to_json()
+    }
+
+    // Updates a record
+    #[export]
+    pub fn update(&mut self, _owner: &Node, uuid: String, model: String, attributes: String) {
+        let new = Record {
+            uuid,
+            model,
+            attributes,
+        };
+
+        let uuid = new.uuid.clone();
+        let original = self.storage.find(|f| &f.uuid, uuid).unwrap().clone();
+
+        self.storage.update(&original, new.clone()).unwrap();
     }
 }
